#!/usr/bin/tclsh 
# $Id: robot.tcl,v 1.20 2001/06/29 22:25:55 adam Exp $
#
proc RobotFileNext1 {area lead} {
    # puts "RobotFileNext1 area=$area lead=$lead"
    if {[catch {set ns [glob ${area}/*]}]} {
        return {}
    }
    foreach n $ns {
	if {[file isfile $n]} {
            set off [string last / $n]
	    incr off 2
            return $lead/[string range $n $off end]
        }
    }
    foreach n $ns {
	if {[file isdirectory $n]} {
            set off [string last / $n]
	    incr off 2
            set sb [RobotFileNext1 $n $lead/[string range $n $off end]]
            if {[string length $sb]} {
                return $sb
            }
        }
    }
    return {}
}

proc RobotWriteRecord {outf fromurl distance} {
    puts $outf "<zmbot>"
    puts $outf "<distance>"
    puts $outf $distance
    puts $outf "</distance>"
    puts $outf "<fromurl>"
    puts $outf $fromurl
    puts $outf "</fromurl>"
    puts $outf "</zmbot>"
}

proc RobotReadRecord {inf fromurlx distancex} {
    upvar $fromurlx fromurl
    upvar $distancex distance
    gets $inf
    gets $inf
    set distance [string trim [gets $inf]]
    # puts "got distance = $distance"
    gets $inf
    gets $inf
    set fromurl [string trim [gets $inf]]
}

proc RobotFileNext {area} {
    global robotSeq global idleTime ns

    # puts "RobotFileNext robotSeq=$robotSeq"
    if {$robotSeq < 0} {
	return {}
    }
    if {$robotSeq == 0} {
	if {[catch {set ns [glob ${area}/*]}]} {
	    return {}
	}
    }
    set off [string length $area]
    incr off
    set n [lindex $ns $robotSeq]
    if {![string length $n]} {
	set robotSeq -1
	flush stdout
	puts "------------ N E X T  R O U N D --------"
	return wait
    }
    incr robotSeq
    if {[file isfile $n/frobots.txt]} {
	puts "ok returning http://[string range $n $off end]/robots.txt"
	return http://[string range $n $off end]/robots.txt
    } elseif {[file isdirectory $n]} {
	set sb [RobotFileNext1 $n http://[string range $n $off end]]
	if {[string length $sb]} {
	    return $sb
	}
    }
    puts "no more work at end of RobotFileNext n=$n"
    puts "ns=$ns"
    return {}
}


proc RobotFileExist {area host path} {
    # puts "RobotFileExist begin area=$area host=$host path=$path"
    set lpath [split $path /]
    set l [llength $lpath]
    incr l -1
    set t [lindex $lpath $l]
    incr l -1
    set npath $area/$host[join [lrange $lpath 0 $l] /d]/f$t
    # puts "RobotFileExist end npath=$npath"
    return [file exists $npath]
}

proc RobotFileUnlink {area host path} {
    # puts "RobotFileUnlink begin"
    # puts "area=$area host=$host path=$path"
    set lpath [split $path /]
    set l [llength $lpath]
    incr l -1
    set t [lindex $lpath $l]
    incr l -1
    set npath $area/$host[join [lrange $lpath 0 $l] /d]/f$t
    # puts "npath=$npath"
    set comp [split $npath /]
    set l [llength $comp]
    incr l -1
    if {[catch {exec rm [join $comp /]}]} return
    incr l -1
    for {set i $l} {$i > 0} {incr i -1} {
        set path [join [lrange $comp 0 $i] /]
	if {![catch {glob $path/*}]} return
        exec rmdir ./$path
    }
    # puts "RobotFileUnlink end"
}

proc RobotFileClose {out} {
    if [string compare $out stdout] {
	close $out
    }
}

proc RobotFileOpen {area host path {mode w}} {
    set orgPwd [pwd]
    global workdir

    if {![info exists workdir]} {
	return stdout
    }
    #puts "RobotFileOpen orgPwd=$orgPwd area=$area host=$host path=$path mode=$mode"
    if {[string compare $orgPwd $workdir]} {
        puts "ooops. RobotFileOpen failed"
	puts "workdir = $workdir"
	puts "pwd = $orgPwd"
	exit 1
    }
    set comp [split $area/$host$path /]
    set len [llength $comp]
    incr len -1
    for {set i 0} {$i < $len} {incr i} {
        if {$i > 1} {
            set d "d[lindex $comp $i]" 
        } else {
            set d [lindex $comp $i]
        }
        if {[catch {cd ./$d}]} {
            exec mkdir $d
            cd ./$d
	    if {![string compare $area unvisited] && $i == 1 && $mode == "w"} {
		set out [open frobots.txt w]
		puts "creating robots.txt in $d"
		close $out
	    }
        }
    }
    set d [lindex $comp $len]
    if {[string length $d]} {
	if {[file isdirectory $d]} {
	    set out [open $d/f $mode]
	} else {
	    set out [open f$d $mode]
	}
    } else {
        set out [open f $mode]
    }
    cd $orgPwd
    return $out
}

proc RobotRR {} {
    global robotSeq robotsRunning

    incr robotsRunning -1
    while {$robotsRunning} {
	vwait robotsRunning
    }
    set robotSeq 0
    RobotStart
}

proc RobotRestart {url sock} {
    global URL robotsRunning

    close $sock
    after cancel $URL($sock,cancel) 

    foreach v [array names URL $url,*] {
	unset URL($v)
    }

    incr robotsRunning -1
    RobotStart
}

proc RobotStart {} {
    global URL
    global robotsRunning robotsMax idleTime
  
    # puts "RobotStart"
    while {1} {
        set url [RobotFileNext unvisited]
        if {![string length $url]} {
	    return
	}
	incr robotsRunning
	if {[string compare $url wait] == 0} {
	    after $idleTime RobotRR
	    return
	}
        set r [RobotGetUrl $url {}]
        if {!$r} {
	    if {$robotsRunning >= $robotsMax} return
        } else {
	    incr robotsRunning -1
	    if {![RobotFileExist bad $URL($url,hostport) $URL($url,path)]} {
		set outf [RobotFileOpen bad $URL($url,hostport) $URL($url,path)]
		RobotFileClose $outf
	    }
            RobotFileUnlink unvisited $URL($url,hostport) $URL($url,path)
	}
    }
}

proc headSave {url out} {
    global URL
    
    if {[info exists URL($url,head,last-modified)]} {
        puts $out "<lastmodified>$URL($url,head,last-modified)</lastmodified>"
    }
    puts $out {<si>}
    if {[info exists URL($url,head,date)]} {
        puts $out " <date>$URL($url,head,date)</date>"
    }
    if {[info exists URL($url,head,content-length)]} {
        puts $out " <by>$URL($url,head,content-length)</by>"
    }
    if {[info exists URL($url,head,server)]} {
        puts $out " <format>$URL($url,head,server)</format>"
    }
    puts $out {</si>}
    puts $out {<publisher>}
    puts $out " <identifier>$url</identifier>"
    if {[info exists URL($url,head,content-type)]} {
        puts $out " <type>$URL($url,head,content-type)</type>"
    }
    puts $out {</publisher>}
}

proc RobotHref {url hrefx hostx pathx} {
    global URL domains
    upvar $hrefx href
    upvar $hostx host
    upvar $pathx path

    puts "Ref url = $url href=$href"

    if {[string first { } $href] >= 0} {
	return 0
    }
    if {[string length $href] > 256} {
	return 0
    }
    if {[string first {?} $href] >= 0} {
	return 0
    }
    if {[string first {?} $url] >= 0 && [string first {?} $href] >= 0} {
	return 0
    }
    # get method (if any)
    if {![regexp {^([^/:]+):(.*)} $href x method hpath]} {
	set hpath $href
	set method http
    } else {
	if {[string compare $method http]} {
	    return 0
	}
    }
    # get host (if any)
    if {[regexp {^//([^/]+)([^\#]*)} $hpath x host surl]} {
	if {![string length $surl]} {
	    set surl /
	}
        if {[info exist domains]} {
	    set ok 0
	    foreach domain $domains {
	        if {[string match $domain $host]} {
		    set ok 1
		    break
	         }
	    }
	    if {!$ok} {
	        return 0
	    }
        }
    } else {
	regexp {^([^\#]*)} $hpath x surl
	set host $URL($url,hostport)
    }
    if {![string length $surl]} {
	return 0
    }
    if {[string first / $surl]} {
	# relative path
	regexp {^([^\#?]*)} $URL($url,path) x dpart
	set l [string last / $dpart]
	if {[expr $l >= 0]} {
	    set surl [string range $dpart 0 $l]$surl
	} else {
	    set surl $dpart/$surl
	}
    }
    set surllist [split $surl /]
    catch {unset path}
    set pathl 0
    foreach c $surllist {
        switch -- $c {
	    .. {
 		if {$pathl > 0} {
		    incr pathl -2
		    set path [lrange $path 0 $pathl]
		    incr pathl
		}
	    }
            . {

            }
            default {
		incr pathl
                lappend path $c
	    }
	}
    }
    if {$pathl} {
	set path [join $path /]
    } else {
	set path ""
    }
    regsub -all {~} $path {%7E} path
    set href "$method://$host$path"
    puts "Ref href = $href"
    return 1
}

proc RobotError {url code} {
    global URL

    puts "Bad URL $url, $code"
    set fromurl {}
    set distance -1
    if {[RobotFileExist unvisited $URL($url,hostport) $URL($url,path)]} {
	set inf [RobotFileOpen unvisited $URL($url,hostport) $URL($url,path) r]
	RobotReadRecord $inf fromurl distance
	RobotFileClose $inf
    }
    RobotFileUnlink unvisited $URL($url,hostport) $URL($url,path)
    if {![RobotFileExist bad $URL($url,hostport) $URL($url,path)]} {
	set outf [RobotFileOpen bad $URL($url,hostport) $URL($url,path)]
	RobotWriteRecord $outf $fromurl $distance
	RobotFileClose $outf
    }
}

proc RobotRedirect {url tourl code} {
    global URL

    puts "Redirecting from $url to $tourl"

    set distance {}
    set fromurl {}
    if {[RobotFileExist unvisited $URL($url,hostport) $URL($url,path)]} {
	set inf [RobotFileOpen unvisited $URL($url,hostport) $URL($url,path) r]
	RobotReadRecord $inf fromurl distance
	RobotFileClose $inf
    }
    if {![RobotFileExist bad $URL($url,hostport) $URL($url,path)]} {
	set outf [RobotFileOpen bad $URL($url,hostport) $URL($url,path)]
	RobotWriteRecord $outf $fromurl $distance
	RobotFileClose $outf
    }
    if {[RobotHref $url tourl host path]} {
	if {![RobotFileExist visited $host $path]} {
	    if {![RobotFileExist unvisited $host $path]} {
		set outf [RobotFileOpen unvisited $host $path]
		RobotWriteRecord $outf $fromurl $distance
		RobotFileClose $outf
	    }
	} else {
	    set olddistance {}
	    set inf [RobotFileOpen visited $host $path r]
	    RobotReadRecord $inf oldurl olddistance
	    RobotFileClose $inf
	    if {[string length $olddistance] == 0} {
		set olddistance 1000
	    }
	    if {[string length $distance] == 0} {
		set distance 1000
	    }
	    puts "distance=$distance olddistance=$olddistance"
	    if {[expr $distance < $olddistance]} {
		set outf [RobotFileOpen unvisited $host $path]
		RobotWriteRecord $outf $tourl $distance
		RobotFileClose $outf
	    }
	}
    }
    if {[catch {RobotFileUnlink unvisited $URL($url,hostport) $URL($url,path)}]} {
        puts "unlink failed"
        exit 1
    }
}

proc RobotTextHtml {url out} {
    global URL maxDistance

    set distance 0
    if {$maxDistance < 1000 && [info exists URL($url,dist)]} {
	set distance [expr $URL($url,dist) + 1]
    }
    htmlSwitch $URL($url,buf) \
        title {
	    puts $out "<title>$body</title>"
        } -nonest meta {
            puts -nonewline $out "<meta"
            foreach a [array names parm] {
		puts -nonewline $out " $a"
                puts -nonewline $out {="}
                puts -nonewline $out $parm($a)
                puts -nonewline $out {"}
            }
	    puts $out {></meta>}
	} body {
	    regsub -all -nocase {<script([^<]|(<!.*>))*</script>} $body {} abody
	    regsub -all {<[^\>]+>} $abody {} nbody
	    puts $out "<documentcontent>"
            puts $out $nbody
            puts $out "</documentcontent>"
        } -nonest a {
            if {![info exists parm(href)]} {
	        puts "no href"
	        continue
            }
	    if {[expr $distance <= $maxDistance]} {
		set href [string trim $parm(href)]
		if {![RobotHref $url href host path]} continue
		
		puts $out "<cr>"
		puts $out "<identifier>$href</identifier>"
		puts $out "<description>$body</description>"
		puts $out "</cr>"

		if {![RobotFileExist visited $host $path]} {
		    set olddistance 1000
		    if {![RobotFileExist bad $host $path]} {
			if {[RobotFileExist unvisited $host $path]} {
			    set inf [RobotFileOpen unvisited $host $path r]
			    RobotReadRecord $inf oldurl olddistance
			    RobotFileClose $inf
			}
		    } else {
			set olddistance 0
		    }
		    if {[string length $olddistance] == 0} {
			set olddistance 1000
		    }
		    if {[expr $distance < $olddistance]} {
			set outf [RobotFileOpen unvisited $host $path]
			RobotWriteRecord $outf $url $distance
			RobotFileClose $outf
		    }
		} elseif {[string compare $href $url]} {
		    set inf [RobotFileOpen visited $host $path r]
		    RobotReadRecord $inf xurl olddistance
		    close $inf
		    if {[string length $olddistance] == 0} {
			set olddistance 1000
		    }
		    if {[expr $distance < $olddistance]} {
			puts "OK remarking url=$url href=$href"
			puts "olddistance = $olddistance"
			puts "newdistance = $distance"
		        set outf [RobotFileOpen unvisited $host $path]
			RobotWriteRecord $outf $url $distance
		        RobotFileClose $outf
		    }
		}
	    }
        } -nonest area {
            if {![info exists parm(href)]} {
	        puts "no href"
	        continue
            }
	    if {[expr $distance <= $maxDistance]} {
		set href [string trim $parm(href)]
		if {![RobotHref $url href host path]} continue
		
		puts $out "<cr>"
		puts $out "<identifier>$href</identifier>"
		puts $out "<description></description>"
		puts $out "</cr>"

		if {![RobotFileExist visited $host $path]} {
		    set olddistance 1000
		    if {![RobotFileExist bad $host $path]} {
			if {[RobotFileExist unvisited $host $path]} {
			    set inf [RobotFileOpen unvisited $host $path r]
			    RobotReadRecord $inf oldurl olddistance
			    RobotFileClose $inf
			}
		    } else {
			set olddistance 0
		    }
		    if {[string length $olddistance] == 0} {
			set olddistance 1000
		    }
		    if {[expr $distance < $olddistance]} {
			set outf [RobotFileOpen unvisited $host $path]
			RobotWriteRecord $outf $url $distance
			RobotFileClose $outf
		    }
		} elseif {[string compare $href $url]} {
		    set inf [RobotFileOpen visited $host $path r]
		    RobotReadRecord $inf xurl olddistance
		    close $inf
		    if {[string length $olddistance] == 0} {
			set olddistance 1000
		    }
		    if {[expr $distance < $olddistance]} {
			puts "OK remarking url=$url href=$href"
			puts "olddistance = $olddistance"
			puts "newdistance = $distance"
		        set outf [RobotFileOpen unvisited $host $path]
			RobotWriteRecord $outf $url $distance
		        RobotFileClose $outf
		    }
		}
	    }
	}
}

proc RobotsTxt {url} {
    global agent URL

    RobotsTxt0 URL(URL($url,hostport),robots) $URL($url,buf)
}

proc RobotsTxt0 {v buf} {
    global URL agent
    set section 0
    foreach l [split $buf \n] {
	if {[regexp {([-A-Za-z]+):[ ]*([^\# ]+)} $l match cmd arg]} {
	    puts "cmd=$cmd arg=$arg"
	    switch -- [string tolower $cmd] {
		user-agent {
		    if {$section} break
		    set pat [string tolower $arg]*
		    set section [string match $pat $agent]
		}
		disallow {
		    if {$section} {
			puts "rule [list 0 $arg]"
			lappend $v [list 0 $arg]
		    }
		}
		allow {
		    if {$section} {
			puts "rule [list 1 $arg]"
			lappend $v [list 1 $arg]
		    }
		}
	    }
	}
    }
}

proc RobotTextPlain {url out} {
    global URL

    puts $out "<documentcontent>"
    regsub -all {<} $URL($url,buf) {\&lt;} content
    puts $out $content
    puts $out "</documentcontent>"

    if {![string compare $URL($url,path) /robots.txt]} {
	RobotsTxt $url
    }
}

proc Robot200 {url} {
    global URL domains
    
    set out [RobotFileOpen raw $URL($url,hostport) $URL($url,path)]
    puts -nonewline $out $URL($url,buf)
    RobotFileClose $out

    set out [RobotFileOpen visited $URL($url,hostport) $URL($url,path)]
    puts $out "<zmbot>"

    set distance 1000
    if {[RobotFileExist unvisited $URL($url,hostport) $URL($url,path)]} {
	set inf [RobotFileOpen unvisited $URL($url,hostport) $URL($url,path) r]
	RobotReadRecord $inf fromurl distance
	RobotFileClose $inf
    }
    set URL($url,dist) $distance
    puts $out "<distance>"
    puts $out "  $distance"
    puts $out "</distance>"
    headSave $url $out
    puts "Parsing $url distance=$distance"
    switch $URL($url,head,content-type) {
	text/html {
	    if {[string length $distance]} {
		RobotTextHtml $url $out
	    }
	}
	text/plain {
	    RobotTextPlain $url $out
	}
	application/pdf {
	    set pdff [open test.pdf w]
	    puts -nonewline $pdff $URL($url,buf)
	    close $pdff
	}
    }
    puts $out "</zmbot>"
    RobotFileClose $out
    # puts "Parsing done"
    RobotFileUnlink unvisited $URL($url,hostport) $URL($url,path)
}

proc RobotReadContent {url sock binary} {
    global URL

    set buffer [read $sock 16384]
    set readCount [string length $buffer]

    if {$readCount <= 0} {
	Robot200 $url
	RobotRestart $url $sock
    } elseif {!$binary && [string first \0 $buffer] >= 0} {
	Robot200 $url
	RobotRestart $url $sock
    } else {
	# puts "Got $readCount bytes"
	set URL($url,buf) $URL($url,buf)$buffer
    }
}

proc RobotReadHeader {url sock} {
    global URL

    puts "RobotReadHeader $url"
    if {[catch {set buffer [read $sock 2148]}]} {
	RobotError $url 404
	RobotRestart $url $sock
    }
    set readCount [string length $buffer]
    
    if {$readCount <= 0} {
	RobotError $url 404
	RobotRestart $url $sock
    } else {
	# puts "Got $readCount bytes"
	set URL($url,buf) $URL($url,buf)$buffer
	
	set n [string first \r\n\r\n $URL($url,buf)]
	if {$n > 1} {
	    set code 0
	    set version {}
	    set headbuf [string range $URL($url,buf) 0 $n]
	    incr n 4
	    set URL($url,buf) [string range $URL($url,buf) $n end]
	    
	    regexp {^HTTP/([0-9.]+)[ ]+([0-9]+)} $headbuf x version code
	    set lines [split $headbuf \n]
	    foreach line $lines {
		if {[regexp {^([^:]+):[ ]+([^;]*)} $line x name value]} {
		    set URL($url,head,[string tolower $name]) [string trim $value]
		}
	    }
	    puts "code = $code"
	    set URL($url,state) skip
	    switch $code {
		301 {
		    RobotRedirect $url $URL($url,head,location) 301
		    RobotRestart $url $sock
		}
		302 {
		    RobotRedirect $url $URL($url,head,location) 302
		    RobotRestart $url $sock
		}
		200 {
		    if {![info exists URL($url,head,content-type)]} {
			set URL($url,head,content-type) {}
		    }
		    set binary 0
		    switch $URL($url,head,content-type) {
			application/pdf {
			    set binary 1
			}
		    }
		    fileevent $sock readable [list RobotReadContent $url $sock $binary]
		}
		default {
		    RobotError $url $code
		    RobotRestart $url $sock
		}
	    }
	}
    }
}

proc RobotSockCancel {url sock} {

    puts "RobotSockCancel sock=$sock url=$url"
    RobotError $url 401
    RobotRestart $url $sock
}

proc RobotConnect {url sock} {
    global URL agent acceptLanguage

    fconfigure $sock -translation {lf crlf} -blocking 0
    fileevent $sock readable [list RobotReadHeader $url $sock]
    puts $sock "GET $URL($url,path) HTTP/1.0"
    puts $sock "Host: $URL($url,host)"
    puts $sock "User-Agent: $agent"
    if {[string length $acceptLanguage]} {
        puts $sock "Accept-Language: $acceptLanguage"
    }
    puts $sock ""
    flush $sock
    set URL($sock,cancel) [after 30000 [list RobotSockCancel $url $sock]]
}

proc RobotNop {} {

}

proc RobotGetUrl {url phost} {
    global URL robotsRunning
    flush stdout
    puts "RobotGetUrl --------- robotsRunning=$robotsRunning url=$url"
    if {![regexp {([^:]+)://([^/]+)(.*)} $url x method hostport path]} {
        return -1
    }
    if {![regexp {([^:]+):([0-9]+)} $hostport x host port]} {
	set port 80
	set host $hostport
    }
    set URL($url,method) $method
    set URL($url,host) $host
    set URL($url,hostport) $hostport
    set URL($url,path) $path
    set URL($url,state) head
    set URL($url,buf) {}

    if {[string compare $path /robots.txt]} {
	set ok 1
	if {![info exists URL($hostport,robots)]} {
	    puts "READING robots.txt for host $hostport"
	    if {[RobotFileExist visited $hostport /robots.txt]} {
		set inf [RobotFileOpen visited $hostport /robots.txt r]
		set buf [read $inf 32768]
		close $inf
	    } else {
		set buf "User-agent: *\nAllow: /\n"
	    }
	    RobotsTxt0 URL($hostport,robots) $buf
	}
	if {[info exists URL($hostport,robots)]} {
	    foreach l $URL($hostport,robots) {
		if {[string first [lindex $l 1] $path] == 0} {
		    set ok [lindex $l 0]
		    break
		}
	    }
	}
	if {!$ok} {
	    puts "skipped due to robots.txt"
	    return -1
	}
    }
    if [catch {set sock [socket -async $host $port]}] {
        return -1
    }
    RobotConnect $url $sock

    return 0
}

if {![llength [info commands htmlSwitch]]} {
    set e [info sharedlibextension]
    if {[catch {load ./tclrobot$e}]} {
	load tclrobot$e
    }
}

set agent "zmbot/0.0"
if {![catch {set os [exec uname -s -r]}]} {
    set agent "$agent ($os)"
}

puts "agent: $agent"

proc bgerror {m} {
    global errorInfo
    puts "BGERROR $m"
    puts $errorInfo
}

set robotsRunning 0
set robotSeq 0
set workdir [pwd]
set idleTime 60000
set acceptLanguage {}

set i 0
set l [llength $argv]

# For testing only
if {0} {
    set url "http://www.sportsfiskeren.dk/sportsfiskeren/corner/index.htm"
    set href "../../data/../../data2/newsovs.asp?Mode=5"

    set URL($url,path) /sportsfiskeren/corner/index.htm
    set URL($url,hostport) www.sportsfiskeren.dk
    RobotHref $url href host path
    exit 0
}

if {$l < 2} {
    puts {tclrobot: usage [-j jobs] [-i idle] [-c count] [-d domain] [url ..]}
    puts " Example: -c 3 -d '*.dk' http://www.indexdata.dk/"
    exit 1
}

while  {$i < $l} {
    set arg [lindex $argv $i]
    switch -glob -- $arg {
	-j* {
	    set robotsMax [string range $arg 2 end]
	    if {![string length $robotsMax]} {
		set robotsMax [lindex $argv [incr i]]
	    }
	}
	-c* {
	    set maxDistance [string range $arg 2 end]
	    if {![string length $maxDistance]} {
		set maxDistance [lindex $argv [incr i]]
	    }
	}
	-d* {
	    set dom [string range $arg 2 end]
	    if {![string length $dom]} {
		set dom [lindex $argv [incr i]]
	    }
	    lappend domains $dom
	}
	-i* {
	    set idleTime [string range $arg 2 end]
	    if {![string length $idleTime]} {
		set idleTime [lindex $argv [incr i]]
	    }
	}
        -l* {
	    set acceptLanguage [string range $arg 2 end]
	    if {![string length $acceptLanguage]} {
		set acceptLanguage [lindex $argv [incr i]]
 	    }
	}
	default {
	    set href $arg
	    if {[RobotHref http://www.indexdata.dk/ href host path]} {
		if {![RobotFileExist visited $host $path]} {
		    set outf [RobotFileOpen unvisited $host $path]
		    RobotWriteRecord $outf href 0
		    RobotFileClose $outf
		}
	    }
	}
    }
    incr i
}

if {![info exist domains]} {
    set domains {*}
}
if {![info exist maxDistance]} {
    set maxDistance 50
}
if {![info exist robotsMax]} {
    set robotsMax 5
}

puts "domains=$domains"
puts "max distance=$maxDistance"
puts "max jobs=$robotsMax"

RobotStart

while {$robotsRunning} {
    vwait robotsRunning
}
